# # pylint: disable=too-many-lines
# # pylint: disable=too-many-locals
# # pylint: disable=too-many-branches
# # pylint: disable=bare-except
# # pylint: disable=line-too-long
# # pylint: disable=unused-import

# '''
#     A module of utility methods used for manipulating directories locally or over FTP.

#     ----------

#     Meta
#     ----------
#     `author`: Colemen Atwood
#     `created`: 06-04-2022 10:44:23
#     `memberOf`: directory_utils
# '''

# # import json
# # from re import search
# # import time
# # import json
# # from threading import Thread
# import traceback as _traceback
# import shutil as _shutil
# import time as _time
# import os as _os
# from pathlib import Path
# import logging as _logging

# import ftputil as _ftputil
# from ftputil.error import FTPOSError as _FTPOSError


# import colemen_utilities.file_utils as _f
# import colemen_utilities.dict_utils as _obj
# import colemen_utilities.string_utils as _csu

# logger = _logging.getLogger(__name__)


# def get_files(search_path=False, **kwargs):
#     return _f.get_files(search_path, **kwargs)


# def create(path, dir_name=False, **kwargs):
#     '''
#         Create a directory or path of directories on the local machine or an FTP server.

#         ----------

#         Arguments
#         -------------------------
#         `path` {str}
#             The path to create or a path to where it should create the dir_name directory
#         [`dir_name`=False] {str}
#             The name of the directory to create.

#         Keyword Arguments
#         -----------------
#         [`ftp`=None] {obj}
#             A reference to the ftputil object.

#         Return {bool}
#         ----------------------
#         True upon success, false otherwise.

#         Meta
#         ----------
#         `author`: Colemen Atwood
#         `created`: 12-19-2021 11:33:50
#         `memberOf`: dir
#         `version`: 1.0
#         `method_name`: create
#     '''
#     success = False
#     ftp = _obj.get_kwarg(["ftp"], None, None, **kwargs)
#     if dir_name is not False:
#         path = _os.path.join(path, dir_name)

#     if ftp is not None:
#         try:
#             if ftp.path.exists(path) is False:
#                 ftp.makedirs(path, exist_ok=True)
#                 success = True
#         except _ftputil.error.PermanentError as error:
#             print(f"error: {str(error)}")
#             print(_traceback.format_exc())
#     else:
#         if exists(path) is False:
#             Path(path).mkdir(parents=True, exist_ok=True)
#             if exists(path) is True:
#                 success = True
#         else:
#             success = True

#     return success


# def exists(file_path, **kwargs):
#     '''
#         Confirms that the directory file_path exists

#         ----------

#         Arguments
#         -------------------------
#         `file_path` {str}
#             The path to confirm.

#         Keyword Arguments
#         -----------------
#         [`ftp`=None] {obj}
#             A reference to the ftputil object.

#         Return {bool}
#         ----------------------
#         True if the directory exists, false otherwise.

#         Meta
#         ----------
#         `author`: Colemen Atwood
#         `created`: 12-19-2021 11:44:03
#         `memberOf`: dir
#         `version`: 1.0
#         `method_name`: exists
#     '''
#     dir_exists = False
#     ftp = _obj.get_kwarg(["ftp"], None, None, **kwargs)

#     if ftp is not None:
#         dir_exists = exists_ftp(file_path, ftp)
#     else:
#         if _os.path.isdir(file_path) is True:
#             dir_exists = True

#     return dir_exists


# def exists_ftp(file_path, ftp):
#     '''
#         Checks if an FTP directory exists.

#         ----------

#         Arguments
#         -------------------------
#         `file_path` {str}
#             The path to confirm.

#         `ftp` {obj}
#             A reference to the ftputil object.

#         Return {bool}
#         ----------------------
#         True if the directory exists, false otherwise.

#         Meta
#         ----------
#         `author`: Colemen Atwood
#         `created`: 12-19-2021 11:59:05
#         `memberOf`: dir
#         `version`: 1.0
#         `method_name`: exists_ftp
#     '''
#     dir_exists = False
#     try:
#         if ftp.path.exists(file_path):
#             dir_exists = True
#     except _ftputil.error.PermanentError as error:
#         print(f"error: {str(error)}")
#     return dir_exists


# def get_folders(search_path=False, **kwargs):
#     '''
#         Get all directories from the search_path.

#         ----------

#         Arguments
#         -------------------------
#         `search_path` {str|list}
#             The search path or list of paths to iterate.\n
#             This is the same as the keyword argument search_path,
#             the kwarg is provided for consistency.

#         Keyword Arguments
#         -----------------
#             [`search_path`=cwd] {str|list}
#                 The search path or list of paths to iterate.

#             [`recursive`=True] {boolean}
#                 If True the path is iterated recursively

#             [`exclude`=[]] {str|list}
#                 A string or list of strings, if the file path contains any of them,
#                 the directory is ignored.

#             [`include`=[]] {str|list}
#                 A string or list of strings, if the file path does NOT contain any of them,
#                 the directory is ignored.

#             [`paths_only`=False] {bool}
#                 If True, the returned value will be a list of directory paths.

#             [`ftp`=None] {obj}
#                 A reference to the ftputil object.

#         Return {list}
#         ----------------------
#         A list of dictionaries containing all matching directories.\n
#         example:\n
#             [{\n
#                 file_path:"beep/boop/bleep/blorp",\n
#                 dir_name:"blorp"\n
#             },...]\n
#         if paths_only = True:\n
#             ["beep/boop/bleep/blorp",...]

#         Meta
#         ----------
#         `author`: Colemen Atwood
#         `created`: 12-19-2021 12:17:24
#         `memberOf`: dir
#         `version`: 1.0
#         `method_name`: get_folders
#     '''
#     dir_array = []
#     if search_path is False:
#         search_path = _obj.get_kwarg(['search path', 'search'], _os.getcwd(), (list, str), **kwargs)
#     if isinstance(search_path, list) is False:
#         search_path = [search_path]

#     recursive = _obj.get_kwarg(['recursive', 'recurse'], True, bool, **kwargs)

#     include = _obj.get_kwarg(['include'], [], (list, str), **kwargs)
#     if isinstance(include, (str)):
#         include = [include]

#     exclude = _obj.get_kwarg(['exclude', 'ignore', 'ignore array'], [], (list, str), **kwargs)
#     if isinstance(exclude, (str)):
#         exclude = [exclude]

#     paths_only = _obj.get_kwarg(['paths only', 'path only'], False, (bool), **kwargs)

#     ftp = _obj.get_kwarg(["ftp"], None, None, **kwargs)
#     if ftp is not None:
#         return get_folders_ftp(search_path, **kwargs)

#     for path in search_path:
#         # # pylint: disable=unused-variable
#         for root, folders, files in _os.walk(path):
#             # print(folders)
#             for current_dir in folders:
#                 if paths_only:
#                     dir_array.append(_os.path.join(root, current_dir))
#                     continue
#                 dir_data = {}
#                 dir_data['dir_name'] = current_dir
#                 dir_data['file_path'] = _os.path.join(root, current_dir)
#                 ignore = False
#                 if len(exclude) > 0:
#                     if _csu.array_in_string(exclude, dir_data['file_path']) is True:
#                         continue
#                 if len(include) > 0:
#                     if _csu.array_in_string(include, dir_data['file_path']) is False:
#                         continue
#                 # if ignore_array is not False:
#                 #     for x in ignore_array:
#                 #         if x in dir_data['file_path']:
#                 #             ignore = True

#                 # if ignore is False:

#                 dir_array.append(dir_data)

#             if recursive is False:
#                 break
#     return dir_array


# def get_folders_ftp(search_path=False, **kwargs):
#     '''
#         Get all directories from the search_path.

#         ----------

#         Arguments
#         -------------------------
#         `search_path` {str|list}
#             The search path or list of paths to iterate.\n
#             This is the same as the keyword argument search_path,
#             the kwarg is provided for consistency.

#         Keyword Arguments
#         -----------------
#             [`search_path`=cwd] {str|list}
#                 The search path or list of paths to iterate.

#             `ftp` {obj}
#                 A reference to the ftputil object.

#             [`recursive`=True] {boolean}
#                 If True the path is iterated recursively

#             [`exclude`=[]] {str|list}
#                 A string or list of strings, if the file path contains any of them,
#                 the directory is ignored.

#             [`include`=[]] {str|list}
#                 A string or list of strings, if the file path does NOT contain any of them,
#                 the directory is ignored.

#             [`paths_only`=False] {bool}
#                 If True, the returned value will be a list of directory paths.


#         Return {list}
#         ----------------------
#         A list of dictionaries containing all matching directories.\n
#         example:\n
#             [{\n
#                 file_path:"beep/boop/bleep/blorp",\n
#                 dir_name:"blorp"\n
#             },...]\n
#         if paths_only = True:\n
#             ["beep/boop/bleep/blorp",...]

#         Meta
#         ----------
#         `author`: Colemen Atwood
#         `created`: 12-19-2021 12:31:27
#         `memberOf`: dir
#         `version`: 1.0
#         `method_name`: get_folders_ftp
#     '''
#     dir_array = []
#     if search_path is False:
#         search_path = _obj.get_kwarg(['search path', 'search'], _os.getcwd(), (list, str), **kwargs)
#     if isinstance(search_path, list) is False:
#         search_path = [search_path]

#     recursive = _obj.get_kwarg(['recursive', 'recurse'], True, bool, **kwargs)

#     include = _obj.get_kwarg(['include'], [], (list, str), **kwargs)
#     if isinstance(include, (str)):
#         include = [include]

#     exclude = _obj.get_kwarg(['exclude', 'ignore', 'ignore array'], [], (list, str), **kwargs)
#     if isinstance(exclude, (str)):
#         exclude = [exclude]

#     paths_only = _obj.get_kwarg(['paths only', 'path only'], False, (bool), **kwargs)

#     ftp = _obj.get_kwarg(["ftp"], None, None, **kwargs)
#     if ftp is None:
#         logger.warning("No FTP obj reference provided.")
#         return False

#     # print(f"search_path: {search_path}")
#     for path in search_path:
#         # # pylint: disable=unused-variable
#         for root, folders, files in ftp.walk(path):
#             # print(folders)
#             for current_dir in folders:
#                 if paths_only:
#                     dir_array.append(_os.path.join(root, current_dir))
#                     continue
#                 dir_data = {}
#                 dir_data['dir_name'] = current_dir
#                 dir_data['file_path'] = _os.path.join(root, current_dir)
#                 ignore = False
#                 if len(exclude) > 0:
#                     if _csu.array_in_string(exclude, dir_data['file_path']) is True:
#                         continue
#                 if len(include) > 0:
#                     if _csu.array_in_string(include, dir_data['file_path']) is False:
#                         continue

#                 dir_array.append(dir_data)

#             if recursive is False:
#                 break
#     return dir_array


# def index_files(start_path, extension_array=None, ignore_array=None, recursive=True):
#     '''
#         Iterates the start_path to find all files within.

#         ----------
#         Arguments
#         -----------------

#             `search_path`=cwd {str|list}
#                 The search path or list of paths to iterate.
#             `ignore`=[] {str|list}
#                 A term or list or terms to ignore if the file path contains any of them.
#             `extensions`=[] {str|list}
#                 An extension or list of extensions that the file must have.
#             `recursive`=True {boolean}
#                 If True the path is iterated recursively

#         return
#         ----------
#         `return` {str}
#             A list of dictionaries containing all matching files.
#     '''
#     if isinstance(extension_array, list) is False:
#         extension_array = []
#     if isinstance(ignore_array, list) is False:
#         ignore_array = []
#     file_array = []
#     # pylint: disable=unused-variable
#     for root, folders, files in _os.walk(start_path):
#         for file in files:
#             file_data = file.get_data(_os.path.join(root, file))
#             ignore = False

#             if len(extension_array) > 0:
#                 if file_data['extension'] not in extension_array:
#                     ignore = True

#             if len(ignore_array) > 0:
#                 for ignore_string in ignore_array:
#                     if ignore_string in file_data['file_path']:
#                         ignore = True

#             if ignore is False:
#                 # fd['file_hash'] = generateFileHash(fd['file_path'])
#                 file_array.append(file_data)

#         if recursive is False:
#             break
#     return file_array

# def persistent_delete(file_path,attempt_limit=100):
#     success = False
#     attempt_count = 0
#     while success is False:
#         attempt_count += 1
#         if attempt_limit == attempt_count:
#             return False
#         success = delete(file_path)
#         _time.sleep(1)
#     return success

# def delete(file_path, ftp=None,persistent=True):
#     '''
#         Deletes a directory from the local machine or FTP server.

#         ----------

#         Arguments
#         -------------------------
#         `file_path` {str}
#             The path of the directory to delete.

#         [`ftp`=None] {obj}
#             A reference to the ftputil object.


#         Return {bool}
#         ----------------------
#         True upon success, false otherwise.

#         Meta
#         ----------
#         `author`: Colemen Atwood
#         `created`: 12-19-2021 11:56:21
#         `memberOf`: dir
#         `version`: 1.0
#         `method_name`: delete
#     '''
#     attempt_limit = 100
#     if persistent is False:
#         attempt_limit = 1

#     for _ in range(attempt_limit):
#         # success = False
#         if ftp is not None:
#             success = delete_ftp(file_path, ftp)
#         else:
#             try:
#                 _shutil.rmtree(file_path)
#                 success = True
#                 return True
#             except OSError as error:
#                 if persistent is False:
#                     logger.warning("Failed to delete local directory: %s", file_path)
#                     logger.warning("Error: %s : %s", file_path, error.strerror)
#                     # success = False
#                 # print("Error: %s : %s" % (file_path, error.strerror))
#         return success


# def delete_ftp(file_path, ftp):
#     '''
#         Deletes a directory on an FTP server.

#         ----------

#         Arguments
#         -------------------------
#         `file_path` {str}
#             The path of the directory to delete.

#         [`ftp`=None] {obj}
#             A reference to the ftputil object.

#         Return {bool}
#         ----------------------
#         True upon success, false otherwise.

#         Meta
#         ----------
#         `author`: Colemen Atwood
#         `created`: 12-19-2021 12:11:53
#         `memberOf`: dir
#         `version`: 1.0
#         `method_name`: delete_ftp
#     '''
#     success = False
#     if exists_ftp(file_path, ftp):
#         # print(f"{file_path} exists.")
#         try:
#             ftp.rmtree(file_path)
#             success = True
#         except _FTPOSError as error:
#             logger.warning("Failed to delete FTP directory: %s", file_path)
#             logger.warning("Error: %s : %s", file_path, error.strerror)
#     else:
#         success = True
#     return success


# def copy(src, dst=False, **kwargs):
#     '''
#         Copy a directory to another location.

#         ----------

#         Arguments
#         -------------------------
#         `src` {str|list|dict}
#             The source directory to copy.\n
#             A list of dictionaries/lists:\n
#                 [["xxx","aaa"],{src_path:"xxx",dst_path:"aaa"}]
#             A dictionary:\n
#                 {src_path:"xxx",dst_path:"aaa"}

#         Keyword Arguments
#         -------------------------
#         `ftp` {obj}
#             A reference to the ftputil object.
#         [`ftp_direction`='local_to_server'] {str}
#             The direction of the copying:
#                 local_to_server: Copy local directories/files to the FTP server.
#                 server_to_local: Copy FTP server directories/files to the Local machine.

#         Return {type}
#         ----------------------
#         return_description

#         Meta
#         ----------
#         `author`: Colemen Atwood
#         `created`: 12-19-2021 12:34:48
#         `memberOf`: dir
#         `version`: 1.0
#         `method_name`: copy
#     '''
#     ftp = _obj.get_kwarg(["ftp"], None, None, **kwargs)
#     ftp_direction = _obj.get_kwarg(["ftp direction"], 'local_to_server', (str), **kwargs)
#     copy_list = [src, dst]
#     if dst is False:
#         copy_list = _parse_copy_data_from_obj(src)

#     for dir_data in copy_list:
#         if ftp is not None:
#             if ftp_direction == 'local_to_server':
#                 mirror_to_server(dir_data['src_path'], dir_data['dst_path'], **kwargs)
#         else:
#             mirror(dir_data['src_path'], dir_data['dst_path'], **kwargs)


# def mirror(src, dst, **kwargs):
#     '''
#         Mirrors a source directory to the destination directory.\n
#         Optionally, copying files.\n

#         ----------

#         Arguments
#         -------------------------
#         `src` {str}
#             The file path to be copied to the dst
#         `dst` {str}
#             The path to copy the src to.

#         Keyword Arguments
#         -------------------------
#         [`empty_files`=False] {bool}
#             If True, files are copied but have no contents.
#         [`dirs_only`=False] {bool}
#             If True, only directories are copied.
#         [`recursive`=True] {bool}
#             If True the path is iterated recursively
#         [`exclude`=[]] {str|list}
#             A string or list of strings, if the file path contains any of them,
#             the directory is ignored.\n
#             If provided, these rules apply to both files and directories.
#         [`include`=[]] {str|list}
#             A string or list of strings, if the file path does NOT contain any of them,
#             the directory is ignored.\n
#             If provided, these rules apply to both files and directories.
#         [`exclude_dirs`=[]] {str|list}
#             A string or list of strings, if the file path contains any of them,
#             the directory is ignored.\n
#             If provided, these rules apply only to directories.
#         [`include_dirs`=[]] {str|list}
#             A string or list of strings, if the file path does NOT contain any of them,
#             the directory is ignored.\n
#             If provided, these rules apply only to directories.
#         [`exclude_files`=[]] {str|list}
#             A string or list of strings, if the file path contains any of them,
#             the directory is ignored.\n
#             If provided, these rules apply only to files.
#         [`include_files`=[]] {str|list}
#             A string or list of strings, if the file path does NOT contain any of them,
#             the directory is ignored.\n
#             If provided, these rules apply only to files.
#         [`ftp`=None] {obj}
#             A reference to the ftputil object.
#         [`ftp_direction`='local_to_server'] {str}
#             The direction of the copying:
#                 local_to_server: Copy local directories/files to the FTP server.
#                 server_to_local: Copy FTP server directories/files to the Local machine.


#         Return {type}
#         ----------------------
#         return_description

#         Meta
#         ----------
#         `author`: Colemen Atwood
#         `created`: 12-11-2021 14:34:12
#         `memberOf`: dir
#         `version`: 1.0
#         `method_name`: mirror
#     '''
#     # if EMPTY_FILES is True, it creates a duplicate file with no content.
#     empty_files = _obj.get_kwarg(['empty files'], False, bool, **kwargs)
#     dirs_only = _obj.get_kwarg(['dirs only'], False, bool, **kwargs)
#     recursive = _obj.get_kwarg(['recursive', 'recurse'], True, (bool), **kwargs)
#     include = _obj.get_kwarg(['include'], [], (list, str), **kwargs)
#     exclude = _obj.get_kwarg(['exclude'], [], (list, str), **kwargs)

#     include_dirs = _obj.get_kwarg(['include dirs'], include, (list, str), **kwargs)
#     exclude_dirs = _obj.get_kwarg(['exclude dirs'], exclude, (list, str), **kwargs)

#     include_files = _obj.get_kwarg(['include files'], include, (list, str), **kwargs)
#     exclude_files = _obj.get_kwarg(['exclude files'], exclude, (list, str), **kwargs)

#     ftp = _obj.get_kwarg(["ftp"], None, None, **kwargs)
#     ftp_direction = _obj.get_kwarg(["ftp direction"], 'local_to_server', (str), **kwargs)

#     if ftp is not None:
#         if ftp_direction == "local_to_server":
#             return mirror_to_server(src, dst, **kwargs)

#     src = _os.path.abspath(src)
#     if exists(src) is False:
#         logger.warning("Source path must exist.\nsource: %s", src)
#         return False

#     if exists(dst) is False:
#         _os.makedirs(dst)
#     dirs = get_folders(search_path=src, recursive=recursive,
#                        include=include_dirs, exclude=exclude_dirs)

#     for folder in dirs:
#         folder['dst_path'] = folder['file_path'].replace(src, dst)
#         try:
#             _os.makedirs(folder['dst_path'], exist_ok=True)
#             if dirs_only is False:
#                 files = _f.get_files(search_path=folder['file_path'], include=include_files,
#                                        exclude=exclude_files, recursive=False)
#                 # newlist = [x['dst_path'] = x['file_path'].replace(src, dst) for x in files]
#                 for file in files:
#                     file['src_path'] = file['file_path']
#                     file['dst_path'] = file['file_path'].replace(src, dst)
#                 # folder['dst_path'] = folder['file_path'].replace(src, dst)
#                 if empty_files is True:
#                     for file in files:
#                         _f.write(file['dst_path'], "EMPTY TEST FILE CONTENT")
#                 else:
#                     _f.copy(files)
#         except:
#             # print(f"{_traceback.format_exc()}")
#             logger.warning("failed to create directory: %s", folder["dst_path"])
#             logger.warning(_traceback.format_exc())


# def mirror_to_server(src, dst, **kwargs):
#     '''
#         Mirrors the local source directory to the FTP destination directory.
#         Optionally, copying files.

#         ----------

#         Arguments
#         -------------------------
#         `src` {str}
#             The LOCAL file path to be copied to the dst
#         `dst` {str}
#             The FTP path to copy the src to.

#         Keyword Arguments
#         -------------------------
#         `ftp` {obj}
#             A reference to the ftputil object.
#         [`dirs_only`=False] {bool}
#             If True, only directories are copied.
#         [`recursive`=True] {bool}
#             If True the path is iterated recursively
#         [`exclude`=[]] {str|list}
#             A string or list of strings, if the file path contains any of them,
#             the directory is ignored.\n
#             If provided, these rules apply to both files and directories.
#         [`include`=[]] {str|list}
#             A string or list of strings, if the file path does NOT contain any of them,
#             the directory is ignored.\n
#             If provided, these rules apply to both files and directories.
#         [`exclude_dirs`=[]] {str|list}
#             A string or list of strings, if the file path contains any of them,
#             the directory is ignored.\n
#             If provided, these rules apply only to directories.
#         [`include_dirs`=[]] {str|list}
#             A string or list of strings, if the file path does NOT contain any of them,
#             the directory is ignored.\n
#             If provided, these rules apply only to directories.
#         [`exclude_files`=[]] {str|list}
#             A string or list of strings, if the file path contains any of them,
#             the directory is ignored.\n
#             If provided, these rules apply only to files.
#         [`include_files`=[]] {str|list}
#             A string or list of strings, if the file path does NOT contain any of them,
#             the directory is ignored.\n
#             If provided, these rules apply only to files.



#         Return {type}
#         ----------------------
#         return_description

#         Meta
#         ----------
#         `author`: Colemen Atwood
#         `created`: 12-11-2021 14:34:12
#         `memberOf`: dir
#         `version`: 1.0
#         `method_name`: mirror
#     '''
#     # if EMPTY_FILES is True, it creates a duplicate file with no content.
#     dirs_only = _obj.get_kwarg(['dirs only'], False, bool, **kwargs)
#     recursive = _obj.get_kwarg(['recursive', 'recurse'], True, (bool), **kwargs)
#     include = _obj.get_kwarg(['include'], [], (list, str), **kwargs)
#     exclude = _obj.get_kwarg(['exclude'], [], (list, str), **kwargs)

#     include_dirs = _obj.get_kwarg(['include dirs'], include, (list, str), **kwargs)
#     exclude_dirs = _obj.get_kwarg(['exclude dirs'], exclude, (list, str), **kwargs)

#     include_files = _obj.get_kwarg(['include files'], include, (list, str), **kwargs)
#     exclude_files = _obj.get_kwarg(['exclude files'], exclude, (list, str), **kwargs)

#     ftp = _obj.get_kwarg(["ftp"], None, None, **kwargs)
#     if ftp is None:
#         logger.warning("No FTP object provided.")
#         return False

#     src = _os.path.abspath(src)
#     if exists(src) is False:
#         logger.warning("Source path must exist.\nsource: %s", src)
#         return False

#     if exists(dst, ftp=ftp) is False:
#         create(dst, ftp=ftp)

#     dirs = get_folders(search_path=src, recursive=recursive,
#                        include=include_dirs, exclude=exclude_dirs, ftp=ftp)

#     for folder in dirs:
#         folder['dst_path'] = folder['file_path'].replace(src, dst)
#         try:
#             create(folder['dst_path'], ftp=ftp)
#             if dirs_only is False:
#                 files = _f.get_files(search_path=folder['file_path'], include=include_files,
#                                        exclude=exclude_files, recursive=False, ftp=ftp)
#                 # newlist = [x['dst_path'] = x['file_path'].replace(src, dst) for x in files]
#                 for file in files:
#                     file['src_path'] = file['file_path']
#                     file['dst_path'] = file['file_path'].replace(src, dst)
#                 _f.copy(files, ftp=ftp)
#         except:
#             logger.warning("failed to create directory: %s", folder["dst_path"])
#             logger.warning(_traceback.format_exc())


# def _parse_copy_data_from_obj(file_obj):
#     data = {
#         "src_path": None,
#         "dst_path": None,
#     }
#     if isinstance(file_obj, (tuple, list)):
#         if len(file_obj) == 2:
#             data['src_path'] = file_obj[0]
#             data['dst_path'] = file_obj[1]
#         else:
#             print("Invalid list/tuple provided for copy file. Must be [source_file_path, destination_file_path]")
#     if isinstance(file_obj, (dict)):
#         for syn in _f.resources.SRC_PATH_SYNONYMS:
#             synvar = _csu.variations(syn)
#             for synonym_variant in synvar:
#                 if synonym_variant in file_obj:
#                     data['src_path'] = file_obj[synonym_variant]
#         for syn in _f.resources.DEST_PATH_SYNONYMS:
#             synvar = _csu.variations(syn)
#             for synonym_variant in synvar:
#                 if synonym_variant in file_obj:
#                     data['dst_path'] = file_obj[synonym_variant]

#     if exists(data['src_path']) is False:
#         print(f"Invalid source path provided, {data['src_path']} could not be found.")
#     return data





# pylint: disable=missing-function-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=line-too-long
# pylint: disable=unused-wildcard-import
# pylint: disable=wildcard-import
# pylint: disable=unused-import
'''
    A module of utility methods used for parsing and converting python types.

    ----------

    Meta
    ----------
    `author`: Colemen Atwood
    `created`: 06-03-2022 10:22:15
    `memberOf`: type_utils
'''
import re
from typing import Union
import colemen_utilities.dict_utils as _obj

def is_email(value:str)->bool:
    if re.match(r'([A-Za-z0-9]+[.-_])*[A-Za-z0-9]+@[A-Za-z0-9-]+(\.[A-Z|a-z]{2,})+',value) is None:
        return False
    return True

def alpha_only(value:str)->bool:
    return False if re.match(r'^[a-zA-Z]*$',value) is None else True


def alphanumeric_only(value:str)->bool:
    return False if re.match(r'^[a-zA-Z0-9]*$',value) is None else True

def numeric_only(value:str)->bool:
    return False if re.match(r'^[0-9]*$',value) is None else True

def phone_number(value:str)->bool:
    return False if re.match(r'^(\+\d{1,2}\s)?\(?\d{3}\)?[\s.-]\d{3}[\s.-]\d{4}$',value) is None else True

# def alpha_only(value:str)->bool:
# def alpha_only(value:str)->bool:

def ip_address(value:Union[str,int])->bool:
    import ipaddress
    try:
        ipaddress.ip_address(value)
        # print("Valid IP Address")
        return True
    except ValueError:
        pass
        # print("Invalid IP Address")
    return False

def future_unix(value:int)->bool:
    '''
        Determine if the value provided is a unix timestamp set in the future.
        ----------


        Return {bool}
        ----------------------
        True upon success, false otherwise.

        Meta
        ----------
        `author`: Colemen Atwood
        `created`: 12-05-2022 13:58:56
        `memberOf`: cerberus
        `version`: 1.0
        `method_name`: future_unix
        * @TODO []: documentation for future_unix
    '''
    import time
    return False if value <= time.time() else True

def past_unix(value:int)->bool:
    '''
        Determine if the value provided is a unix timestamp set in the past.
        ----------


        Return {bool}
        ----------------------
        True upon success, false otherwise.

        Meta
        ----------
        `author`: Colemen Atwood
        `created`: 12-05-2022 13:58:56
        `memberOf`: cerberus
        `version`: 1.0
        `method_name`: past_unix
        * @TODO []: documentation for past_unix
    '''
    import time
    return False if value >= time.time() else True

def to_hash_id(value:str,prefix:str):
    if prefix not in value:
        value= f"{prefix}_{value}"

def crud_type(value:str):
    valids = ["create","read","update","delete"]
    if value.lower() not in valids:
        return False
    return True





# pylint: disable=missing-function-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=line-too-long
# pylint: disable=unused-import
'''
    A module of utility methods used manipulating lists.

    ----------

    Meta
    ----------
    `author`: Colemen Atwood
    `created`: 06-03-2022 10:22:15
    `memberOf`: list_utils
'''

from typing import Iterable as _Iterable
import colemen_utilities.dict_utils as _obj

from colemen_utilities.string_utils.string_conversion import array_to_string_list as to_string_list
from colemen_utilities.string_utils.string_format import array_replace_string as replace_from_list
from colemen_utilities.string_utils.string_format import array_in_string as find_in_string



def append(base=None,value=None,**kwargs):
    '''
        Append an item to the base list.
        This is a lazy way of merging lists or appending a single item.

        ----------

        Arguments
        -------------------------
        `base` {list}
            The list to append an item to.
        `value` {any}
            The value to append to the base.

        Keyword Arguments
        -------------------------
        [`skip_null`=True] {bool}
            if True and the value is None, it will not append it.

        Return {type}
        ----------------------
        return_description

        Meta
        ----------
        `author`: Colemen Atwood
        `created`: 06-01-2022 08:45:33
        `memberOf`: objectUtils
        `version`: 1.0
        `method_name`: append
        # @TODO []: documentation for append
    '''

    if base is None:
        base = []

    skip_null = _obj.get_kwarg(["skip_null"],True,(bool),**kwargs)
    if skip_null is True:
        if value is None:
            return base

    if isinstance(value,(list)):
        base = base + value
    else:
        base.append(value)
    return base

def strip_list_nulls(value:list)->list:
    '''
        Strip None values from a list.

        ----------

        Arguments
        -------------------------
        `value` {list}
            The list to filter None values from.

        Return {list}
        ----------------------
        The list with all None values removed.

        Meta
        ----------
        `author`: Colemen Atwood
        `created`: 06-03-2022 08:38:50
        `memberOf`: objectUtils
        `version`: 1.0
        `method_name`: strip_list_nulls
        * @xxx [06-03-2022 08:39:37]: documentation for strip_list_nulls
    '''


    if isinstance(value,(list)) is False:
        return value
    return [x for x in value if x is not None]

def find_list_diff(list_one, list_two):
    '''
        find elements in list_one that do not exist in list_two.
        @param {list} list_one the primary list for comparison
        @param {list} list_two
        @function findListDiff
    '''
    return [x for x in list_one if x not in list_two]

def force_list(value)->list:
    '''
        Confirm that the value is a list, if not wrap it in a list.

        ----------

        Arguments
        -------------------------
        `value` {any}
            The value to test.

        Return {list}
        ----------------------
        The value as a list

        Meta
        ----------
        `author`: Colemen Atwood
        `created`: 06-03-2022 09:13:57
        `memberOf`: object_utils
        `version`: 1.0
        `method_name`: force_list
        * @xxx [06-03-2022 09:14:52]: documentation for force_list
    '''


    if isinstance(value,(tuple)) is True:
        return list(value)
    if isinstance(value,(list)) is False:
        return [value]
    return value

def count(subj:list,value:any)->int:
    '''
        Count how many times a value occurs in a list.
        This is case sensitive, it will only count exact matches.

        ----------

        Arguments
        -------------------------
        `subj` {list}
            The list to search.

        `value` {any}
            The value to search for.

        Return {int}
        ----------------------
        The number of times the value occurs in the list.

        Meta
        ----------
        `author`: Colemen Atwood
        `created`: 06-07-2022 15:43:26
        `memberOf`: list_utils
        `version`: 1.0
        `method_name`: count
        * @xxx [06-07-2022 15:44:46]: documentation for count
    '''


    return len([x for x in subj if x == value])

def longest_string(arr:_Iterable[str])->tuple:
    '''
        Get the longest string in a list.

        ----------

        Arguments
        -------------------------
        `arr` {list}
            The list of strings to search within.

        Return {tuple}
        ----------------------
        If all strings are the same length, it will return the first one.
        A tuple containing the longest length and its value.\n
        (19,"kitties and titties")

        If no strings are found in the list it will return this tuple:\n
        (0,None)


        Meta
        ----------
        `author`: Colemen Atwood
        `created`: 06-10-2022 06:16:30
        `memberOf`: list_utils
        `version`: 1.0
        `method_name`: longest_string
        * @xxx [06-10-2022 06:19:29]: documentation for longest_string
    '''


    longest_len = 0
    longest_val = None
    for val in arr:
        if isinstance(val,(str)):
            val_len = len(val)
            if val_len > longest_len:
                longest_len = val_len
                longest_val = val
    return (longest_len,longest_val)

def remove_duplicates(arr:list)->_Iterable:
    '''
        Remove duplicate indices from a list.

        ----------

        Arguments
        -------------------------
        `arr` {list}
            The list to filter

        Return {list}
        ----------------------
        The list with duplicates removed.

        Meta
        ----------
        `author`: Colemen Atwood
        `created`: 07-04-2022 11:07:51
        `memberOf`: list_utils
        `version`: 1.0
        `method_name`: remove_duplicates
        * @xxx [07-04-2022 11:08:44]: documentation for remove_duplicates
    '''


    new_list = []
    for val in arr:
        if val not in new_list:
            new_list.append(val)
    return new_list

def lists_to_dict(keys:list,values:list):
    data = {}

    for idx,key in enumerate(keys):
        if idx <= len(values):
            data[key] = values[idx]
        else:
            data[key] = None
    return data

def chunk_list(arr:list,chunk_size=100):
    '''
        Chunk a list into even sub-lists.
        Any remaining elements are appending to the last list.

        ----------

        Arguments
        -------------------------
        `arr` {list}
            The list to chunk
        `chunk_size` {int}
            How many elements each sub-list should have.

        Return {list}
        ----------------------
        A 2D list of elements.


        Example
        ----------------------

        chunk_list([1,2,3,4,5,6,7,8,9],2)

        [
            [1,2],

            [3,4],

            [5,6],

            [7,8,9],
        ]

        Meta
        ----------
        `author`: Colemen Atwood
        `created`: 07-04-2022 11:07:51
        `memberOf`: list_utils
        `version`: 1.0
        `method_name`: remove_duplicates
        * @xxx [07-04-2022 11:08:44]: documentation for remove_duplicates
    '''

    chunks = [arr[x:x+chunk_size] for x in range(0, len(arr), chunk_size)]
    return chunks


# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-branches
# pylint: disable=bare-except
# pylint: disable=line-too-long
'''
    Utility methods for compressing and zipping directories.

    ----------

    Meta
    ----------
    `author`: Colemen Atwood
    `created`: 06-04-2022 10:34:00
    `memberOf`: directory_utils
'''


import shutil as _shutil
import colemen_utilities.file_utils as _f
import colemen_utilities.dict_utils as _obj
import colemen_utilities.directory_utils as _dir

# logger = logging.getLogger(__name__)

def create_zip(src,dst,**kwargs):
    '''
        Create a zip archive of the directory provided.

        ----------

        Arguments
        -------------------------
        `src` {string}
            The file path to the directory to be archived.

        `dst` {str}
            The file path to the zip file to be created.\
            `WITHOUT AN EXTENSION`

        Keyword Arguments
        -------------------------
        [`delete_after`=False] {bool}
            If True, the source directy will be deleted after the zip is created.

        [`overwrite`=True] {bool}
            If False, it will skip creating the archive.

        Return {void}
        ----------------------
        returns nothing

        Meta
        ----------
        `author`: Colemen Atwood
        `created`: 03\26\2022 11:15:20
        `memberOf`: dir_compression
        `version`: 1.0
        `method_name`: create_zip
        # @xxx [03\26\2022 11:31:44]: documentation for create_zip
    '''

    delete_after = _obj.get_kwarg(["delete after"], False, (bool), **kwargs)
    overwrite = _obj.get_kwarg(["overwrite"], True, (bool), **kwargs)

    if _f.exists(dst):
        if overwrite is False:
            return True

    try:
        result = _shutil.make_archive(dst, 'zip', src)
    except:
        print("Failed to create zip archive.")
        return False
    else:
        if result == dst:
            if delete_after:
                _dir.delete(src)
            return True


# pylint: disable=missing-function-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=line-too-long
# pylint: disable=unused-wildcard-import
# pylint: disable=wildcard-import
# pylint: disable=unused-import
# '''
#     Contains the general methods for manipulating files.

#     ----------

#     Meta
#     ----------
#     `author`: Colemen Atwood
#     `created`: 06-03-2022 10:22:15
#     `memberOf`: rand
#     `version`: 1.0
#     `method_name`: rand
# '''




import json
# import shutil
# import time
# import json
# import re
# from pathlib import Path
import time as _time
import json as _json
from datetime import timezone as _timezone
from datetime import datetime as _datetime
import gzip as _gzip
# import zipfile
import os as _os
import io as _io
import shutil as _shutil
import traceback as _traceback
from threading import Thread as _Thread
import logging as _logging
from pathlib import Path, PureWindowsPath

from isort import file

# import ftputil as _ftputil
# from secure_delete import secure_delete as _secure_delete
# import patoolib
# import colemen_string_utils as strUtils
# import colemen_utilities.files.dir as directory
# import _f.resources
# import colemen_utilities.files.dir as dirUtils
# from colemen_utilities.files.dir import create as create_dir
# import colemen_utilities.object_utils as obj
# import colemen_utilities.files.file_read as read
# import colemen_utilities.files.file_write as write
# import colemen_utilities.files.file_search as search
# import colemen_utilities.files.file_convert as convert
# import colemen_utilities.files.file_image as image
# import colemen_utilities.files.file_compression as compression
# import colemen_utilities.files.exiftool as exiftool
# from colemen_utilities.files.dir import get_folders_ftp
# import colemen_utilities.files.string_utils as strUtils

import colemen_utilities.string_utils as _csu
import colemen_utilities.dict_utils as _obj
import colemen_utilities.file_utils as _f
import colemen_utilities.directory_utils as _directory

# from colemen_utilities.string_utils.string_format import array_in_string as _csu.array_in_string
# from colemen_utilities.string_utils.string_format import file_path as _csu.file_path
# from colemen_utilities.string_utils.string_format import extension as _csu.extension
# from colemen_utilities.string_utils.string_generation import variations as _csu.variations
# from colemen_utilities.string_utils.string_generation import to_hash as _csu.to_hash
# from colemen_utilities.dict_utils.dict_utils import get_kwarg as _obj.get_kwarg


logger = _logging.getLogger(__name__)






class File():
    def __init__(self,file_path,args:dict=None):
        self.args = {} if args is None else args
        self.data = {
            "file_path":file_path,
            "file_name":None,
            "extension":None,
            "name_no_ext":None,
            "dir_path":None,
            "access_time":None,
            "modified_time":None,
            "created_time":None,
            "size":None,
            "is_json":None,
        }
        self.settings = {}
        self.data = _obj.set_defaults(self.data,self.args)

    @property
    def to_dict(self):
        '''
            Get this File's data as a dictionary.

            `default`:None


            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 07-25-2022 14:48:25
            `@memberOf`: File
            `@property`: to_dict
        '''
        data = {
            "file_path":self.file_path,
            "name":self.name,
            "extension":self.extension,
            "name_no_ext":self.name_no_ext,
            "dir_path":self.dir_path,
            "access_time":self.access_time,
            "modified_time":self.modified_time,
            "created_time":self.created_time,
            "drive":self.drive,
            "size":self.size,
        }
        return data

    @property
    def file_path(self):
        '''
            Get this File's file_path

            `default`:None


            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 07-25-2022 14:37:48
            `@memberOf`: File
            `@property`: file_path
        '''
        path = _obj.get_arg(self.data,['file_path'],None,(str))
        if isinstance(path,(str)):
            path = str(PureWindowsPath(Path(path)))
        return path
    path = file_path

    @property
    def name(self):
        '''
            Get this File's name

            `default`:None


            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 07-25-2022 14:37:33
            `@memberOf`: File
            `@property`: name
        '''
        value = _obj.get_arg(self.data,['name'],None,(str))
        # @Mstep [IF] if the property is not currenty set
        if value is None:
            value = _os.path.basename(self.file_path)
            self.data['name'] = value
        return value
    file_name = name

    @property
    def extension(self):
        '''
            Get this File's extension

            `default`:None

            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 07-25-2022 14:41:26
            `@memberOf`: File
            `@property`: extension
        '''
        value = _obj.get_arg(self.data,['extension'],None,(str))
        # @Mstep [IF] if the property is not currenty set
        if value is None:
            value = _f.get_ext(self.file_path)
            if isinstance(value,(str)):
                self.data['extension'] = value
        return value
    ext = extension

    @extension.setter
    def extension(self,value):
        '''
            Set the File's extension property

            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 08-15-2022 09:40:40
            `@memberOf`: File
            `@property`: extension
        '''
        value = _csu.format_extension(value)
        self.data['extension'] = f".{value}"
    
    @property
    def name_no_ext(self):
        '''
            Get this File's name_no_ext

            `default`:""


            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 07-25-2022 14:43:19
            `@memberOf`: File
            `@property`: name_no_ext
        '''
        value = _obj.get_arg(self.data,['name_no_ext'],None,(str))
        # @Mstep [IF] if the property is not currenty set
        if value is None:
            value = _f.get_name_no_ext(self.file_path)
            self.data['name_no_ext'] = value
        return value

    @property
    def dir_path(self):
        '''
            Get this File's dir_path

            `default`:None


            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 07-25-2022 14:44:32
            `@memberOf`: File
            `@property`: dir_path
        '''
        value = _obj.get_arg(self.data,['dir_path'],None,(str))
        # @Mstep [IF] if the property is not currenty set
        if value is None:
            value = _os.path.dirname(self.file_path)
            self.data['dir_path'] = value
        return value

    @property
    def access_time(self):
        '''
            Get this File's access_time

            `default`:None


            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 07-25-2022 14:44:58
            `@memberOf`: File
            `@property`: access_time
        '''
        value = _obj.get_arg(self.data,['access_time'],None,(int))
        # @Mstep [IF] if the property is not currenty set
        if value is None:
            value = _f.get_access_time(self.file_path)
            self.data['access_time'] = value
        return value

    @property
    def created_time(self):
        '''
            Get this File's created_time

            `default`:None


            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 07-25-2022 14:45:32
            `@memberOf`: File
            `@property`: created_time
        '''
        value = _obj.get_arg(self.data,['created_time'],None,(int))
        # @Mstep [IF] if the property is not currenty set
        if value is None:
            value = _f.get_create_time(self.file_path)
            self.data['created_time'] = value
        return value

    @property
    def modified_time(self):
        '''
            Get this File's modified_time

            `default`:None


            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 07-25-2022 14:46:25
            `@memberOf`: File
            `@property`: modified_time
        '''
        value = _obj.get_arg(self.data,['modified_time'],None,(int))
        # @Mstep [IF] if the property is not currenty set
        if value is None:
            value = _f.get_modified_time(self.file_path)
            self.data['modified_time'] = value
        return value

    @property
    def drive(self):
        '''
            Get this File's drive

            `default`:None


            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 07-25-2022 14:47:02
            `@memberOf`: File
            `@property`: drive
        '''
        value = _obj.get_arg(self.data,['drive'],None,(str))
        # @Mstep [IF] if the property is not currenty set
        if value is None:
            value = _f.get_drive(self.file_path)
            self.data['drive'] = value
        return value

    @property
    def size(self):
        '''
            Get this File's size

            `default`:None


            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 07-25-2022 14:47:34
            `@memberOf`: File
            `@property`: size
        '''
        value = _obj.get_arg(self.data,['size'],None,(int))
        # @Mstep [IF] if the property is not currenty set
        if value is None:
            value = _os.path.getsize(self.file_path)
            self.data['size'] = value
        return value

    @property
    def exists(self):
        '''
            Check if this file actually exists.

            `default`:False


            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 08-15-2022 09:05:54
            `@memberOf`: File
            `@property`: exists
        '''
        value = _obj.get_arg(self.data,['exists'],None,(bool))
        # @Mstep [IF] if the property is not currenty set
        if value is None:
            value = _f.exists(self.file_path)
            self.data['exists'] = value
        return value

    @property
    def _is_json(self):
        '''
            Get this File's _is_json

            If the is_json argument was not provided, this will check the file extension.

            If the extension is not json or jsonc, it will return False.

            `default`:False


            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 08-15-2022 09:37:53
            `@memberOf`: File
            `@property`: _is_json
        '''
        value = _obj.get_arg(self.data,['is_json'],None,(bool))
        # @Mstep [IF] if the property is not currenty set
        if value is None:
            value = False
            if self.extension in ['.json','.jsonc']:
                value = True
            if self.content is not None:
                result = _f.as_json(self.content)
                if result is not False:
                    value = True
                    self.content = result
            self.data['_is_json'] = value
        return value

    @property
    def content(self):
        '''
            Get this File's content

            `default`:None


            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 08-15-2022 09:27:31
            `@memberOf`: File
            `@property`: content
        '''
        value = _obj.get_arg(self.data,['content'],None)
        # @Mstep [IF] if the property is not currenty set
        if value is None:
            value = None
            if self.exists is True:
                value = _f.readr(self.path)
            self.data['content'] = value
        return value

    def content_to_string(self,value):
        value = self.content
        if isinstance(value,(str)) is True:
            return value
        if isinstance(value,(dict,list)):
            # TODO []: add error catching.
            value = json.encode(value)
            return value

    @content.setter
    def content(self,value='',save=True):
        '''
            Set the File's contents property

            `default`:''


            Meta
            ----------
            `@author`: Colemen Atwood
            `@created`: 08-15-2022 09:33:56
            `@memberOf`: File
            `@property`: contents
        '''
        self.data['content'] = value
        if save == True:
            _f.write(self.path,self.content)

    # def write(self,content):
    #     self.data['content'] = 


    def save(self):
        _f.write(self.path,self.contents)

    def append(self,new_contents):
        _f.append(self.path,new_contents)
        # @Mstep [IF] if the contents have already been read.
        if self.content is not None:
            # @Mstep [] set the contents to None
            self.data['content'] = None
            # @Mstep [] read the file again.
            _ = self.content

def file_from_path(file_path,**kwargs):
    
    args = {
        "is_json":_obj.get_kwarg(['is_json','json'],None,(bool),**kwargs),
    }
    return File(file_path,args)






# pylint: disable=missing-function-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=line-too-long
# pylint: disable=unused-import

import re

import colemen_utilities.string_utils as _csu
import colemen_utilities.dict_utils as _obj
import yaml












def parse_comment_yaml(contents:str):

    # contents = contents.replace("\n","   ")
    contents = contents.replace("desc:","description:")
    contents = contents.replace("opts:","options:")
    contents = contents.replace("o:","options:")

    # contents = contents.replace("options:","options:\n")



    if len(contents) > 0:
        if contents.startswith("description:") is False:
            contents = f"description: {contents}"

        if "description:" not in contents:
            contents = f"description: {contents}"
    else:
        return None

    # @Mstep [] force a space between a dash and alphanum characters.
    contents = re.sub(r"\n-([a-zA-Z0-9])",r"\n- \1",contents)

    # c.con.log(f"contents: {contents}","red")
    contents = contents.replace("__%0A__","\r\n")
    data = yaml.safe_load(contents)
    output = {}
    if "description" in data:
        output['description'] = data['description']

    if "options" in data:
        # output['options'] = data['options']
        if isinstance(data['options'],(dict)):
            flattend = _obj.flatten(data['options'],'','_')
            flattend = _obj.keys_to_snake_case(flattend)
            # print("\n\n\n")
            # print(flattend)
            # print("\n\n\n")
            output = {**output,**flattend}
            return output
        if data['options'] is not None:
            for o in data['options']:
                if isinstance(o,(str)):
                    output[_csu.to_snake_case(o)] = True
                if isinstance(o,(dict)):
                    for k,v in o.items():
                        # k = _csu.to_snake_case(k)
                        output[_csu.to_snake_case(k)] = v
    # print(output)
    return output


def sql_type_to_python_type(value:str)->str:
    '''
        Convert an SQL type to its PHP equivalent.
        ----------

        Arguments
        -------------------------
        `value` {str}
            The SQL type to convert.


        Return {str}
        ----------------------
        The converted type string, or the original string if no conversion occurred.

        Meta
        ----------
        `author`: Colemen Atwood
        `created`: 11-27-2022 19:23:42
        `memberOf`: __init__
        `version`: 1.0
        `method_name`: sql_type_to_python_type
        * @xxx [11-27-2022 19:24:14]: documentation for sql_type_to_python_type
    '''
    if value in ["decimal","float"]:
        return "float"
    elif value in ["bigint","int","integer"]:
        return "integer"
    elif value in ["tinyint"]:
        return "boolean"
    elif value in ["varchar"]:
        return "string"
    elif value in ["timestamp"]:
        return "string"
    else:
        return value










